package com.Kettle.test;

import org.pentaho.di.core.database.DatabaseMeta;
import org.pentaho.di.core.plugins.PluginRegistry;
import org.pentaho.di.core.plugins.StepPluginType;
import org.pentaho.di.trans.TransMeta;
import org.pentaho.di.trans.step.StepMeta;
import org.pentaho.di.trans.steps.insertupdate.InsertUpdateMeta;
import org.pentaho.di.trans.steps.joinrows.JoinRowsMeta;
import org.pentaho.di.trans.steps.sql.ExecSQLMeta;
import org.pentaho.di.trans.steps.tableinput.TableInputMeta;
import org.pentaho.di.trans.steps.tableoutput.TableOutputMeta;

/**
 * @ClassName TransOrganization
 * @Description TODO Kettle 转换组件创建方法
 * @Author Echo
 * @Date 2021/10/19 16:30
 * @Version 1.0
 */

public class TransOrganization {
    //参数获取类
    //GetParameter getParameter = new GetParameter();
    /**
     * @Author zhh
     * @Description //TODO 表输入组件 * @Date 19:54 2021/10/19
     * @Param [tableInputMeta, selectSql]
     * @return org.pentaho.di.trans.steps.tableinput.TableInputMeta
     **/
    public StepMeta addTableInputMeta(TransMeta transMeta, PluginRegistry registry,int x,int y,String selectSql,String stepName,String databaseName){
        TableInputMeta tableInputMeta = new TableInputMeta();
        //添加数据库连接  只有退回（operationStatus=4）的时候表输入是目标库其他都是源库
        DatabaseMeta database  = transMeta.findDatabase(databaseName);
        tableInputMeta.setDatabaseMeta(database);
        //添加查询语句
        tableInputMeta.setSQL(selectSql);
        //允许替换Sql中的变量
        tableInputMeta.setVariableReplacementActive(true);
        String tableInputPluginId = registry.getPluginId(StepPluginType.class, tableInputMeta);
        //添加TableInputMeta到转换中
        StepMeta tableInputMetaStep = new StepMeta(tableInputPluginId,stepName,tableInputMeta);
        //给步骤添加在spoon工具中的显示位置
        tableInputMetaStep.setDraw(true);
        tableInputMetaStep.setLocation(x, y);
        transMeta.addStep(tableInputMetaStep);
        return tableInputMetaStep;
    }

/**
 * @Author zhh
 * @Description //TODO 表输出组件 * @Date 20:39 2021/10/19
 * @Param [tableOutputMeta  -表输出元,
 *          TableName       -查询表名（一般为源表名称）,
 *          fieldDatabase   -表字段,
 *          fieldStream     -流字段,
 *          transMeta       -转换元,
 *          registry        -生成唯一编号,
 *          x               -spoon工具中的显示位置,
 *          y,              -spoon工具中的显示位置
 *          stepName        -步骤名称,
 *          databaseName    -数据库连接名称
 *          ]
 * @return org.pentaho.di.trans.steps.tableoutput.TableOutputMeta
 **/
    public StepMeta addTableOutput(String tableName ,TransMeta transMeta, PluginRegistry registry,int x,int y,String DatabaseName,String SchemaName,String [] fieldDatabase,String [] fieldStream,String stepName){
        TableOutputMeta tableOutputMeta = new TableOutputMeta();
        //在转换内查找数据库连接
        DatabaseMeta database = transMeta.findDatabase(DatabaseName);
        //设置数据库模式名
        tableOutputMeta.setSchemaName(SchemaName);
//        设置需要修改的表
        tableOutputMeta.setTableName(tableName);
        tableOutputMeta.setDatabaseMeta(database);
        //设置指定表字段
        tableOutputMeta.setSpecifyFields(true);
//        包含要插入的字段名称的字段
        tableOutputMeta.setFieldDatabase(fieldDatabase);
//        包含输入流中要插入到表中的值的字段。
        tableOutputMeta.setFieldStream(fieldStream);
        String tableInputPluginId = registry.getPluginId(StepPluginType.class, tableOutputMeta);
        //添加TableInputMeta到转换中
        StepMeta tableOutputMetaStep = new StepMeta(tableInputPluginId,stepName,tableOutputMeta);
        //给步骤添加在spoon工具中的显示位置
        tableOutputMetaStep.setDraw(true);
        tableOutputMetaStep.setLocation(x, y);
        transMeta.addStep(tableOutputMetaStep);
        return tableOutputMetaStep;
    }

/**
 * @Author zhh
 * @Description //TODO 插入更新组件 * @Date 21:33 2021/10/19
 * @Param [transMeta            -转换,
 *         databaseName         -数据连接名称,
 *         insertUpdateMeta     -插入更新组件,
 *         setTableName         -更改的表名,
 *         KeyLookup            -关键字_表字段,
 *         Stream               -关键字_流字段1 ,
 *         Stream2              -关键字_流字段2,
 *         Condition            -关键字_操作符,
 *         updateLookup         -更新字段_表字段,
 *         updateStream         -更新字段_流字段,
 *         updateOrNot          -更新字段_是否更新,
 *         registry             -生成唯一编号,
 *         stepName             -组件名称,
 *         x                    -位置 x,
 *         y                    -位置 y,]
 * @return org.pentaho.di.trans.steps.insertupdate.InsertUpdateMeta
 **/
    public StepMeta addInsertUpdateMate(TransMeta transMeta,PluginRegistry registry,int x,int y,String stepName,String databaseName,String tableName,
                                        String [] keyLookup,String [] Stream,String [] Stream2,String [] updateLookup,String [] updateStream,String SchemaName){
        InsertUpdateMeta insertUpdateMeta = new InsertUpdateMeta();
        String insertUpdateMetaPluginId = registry.getPluginId(StepPluginType.class,insertUpdateMeta);
        //添加数据库连接
        DatabaseMeta database = transMeta.findDatabase(databaseName);
        insertUpdateMeta.setDatabaseMeta(database);

        //设置操作的表
        insertUpdateMeta.setTableName(tableName);
        //设置数据库模式名
        insertUpdateMeta.setSchemaName(SchemaName);
        //设置用来查询的关键字
        insertUpdateMeta.setKeyLookup(keyLookup);
        insertUpdateMeta.setKeyStream(Stream);
//        insertUpdateMeta.setKeyStream2(Stream2);
        String  [] condition = new String[keyLookup.length];
        String  [] Stream2New = new String[keyLookup.length];
        for(int i = 0;i<keyLookup.length;i++){
            condition[i] = "=";
            Stream2New[i] = "";
        }
        insertUpdateMeta.setKeyStream2(Stream2New);
        insertUpdateMeta.setKeyCondition(condition);

        //设置要更新的字段
        insertUpdateMeta.setUpdateLookup(updateLookup);
        insertUpdateMeta.setUpdateStream(updateStream);
        Boolean [] updateOrNot = new Boolean[updateStream.length];
        for(int i = 0;i<updateStream.length;i++){
            updateOrNot[i] = true;
        }
        insertUpdateMeta.setUpdate(updateOrNot);
        //添加步骤到转换中
        StepMeta insertUpdateStep = new StepMeta(insertUpdateMetaPluginId,stepName,insertUpdateMeta);
        insertUpdateStep.setDraw(true);
        insertUpdateStep.setLocation(x,y);
        transMeta.addStep(insertUpdateStep);
        return insertUpdateStep;
    }

    //执行Sql脚本Mate
    public StepMeta addExecSQLMeta(TransMeta transMeta,PluginRegistry registry ,int x,int y,String databaseName,String updateSql,String [] arguments,String stepName){
        ExecSQLMeta execSQLMeta = new ExecSQLMeta();
        execSQLMeta.setDefault();//默认值 不能少
        DatabaseMeta database = transMeta.findDatabase(databaseName);
        execSQLMeta.setDatabaseMeta(database);
        execSQLMeta.setSql(updateSql);
        execSQLMeta.setVariableReplacementActive(true); //设置“变量替换”
        execSQLMeta.setExecutedEachInputRow(true); //设置“执行每一行”
        if(arguments!=null){ //变量字段
            execSQLMeta.setArguments(arguments);
        }
        String execSQLMetaMetaPluginId = registry.getPluginId(StepPluginType.class,execSQLMeta);
        StepMeta execSQLMetaStep = new StepMeta(execSQLMetaMetaPluginId,stepName,execSQLMeta);
        //给步骤添加在spoon工具中的显示位置
        execSQLMetaStep.setDraw(true);

        execSQLMetaStep.setLocation(x, y);
        transMeta.addStep(execSQLMetaStep);
        return execSQLMetaStep;
    }

    public StepMeta addJoinRowsMeta(TransMeta transMeta,PluginRegistry registry ,int x,int y){
        String stepName = "记录关联";
        JoinRowsMeta joinRowsMeta = new JoinRowsMeta();
        joinRowsMeta.setDefault();
        String JoinRowsMetaPluginId = registry.getPluginId(StepPluginType.class,joinRowsMeta);
        StepMeta JoinRowsMetaStep = new StepMeta(JoinRowsMetaPluginId,stepName,joinRowsMeta);
        JoinRowsMetaStep.setDraw(true);
        JoinRowsMetaStep.setLocation(x, y);
        transMeta.addStep(JoinRowsMetaStep);
        return JoinRowsMetaStep;
    }
}
